/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.quartz;

import java.text.ParseException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.listener.AbstractExternalListener;
import org.ow2.petals.se.quartz.job.EmitSignalJob;
import org.ow2.petals.se.quartz.utils.QuartzConstants;
import org.ow2.petals.se.quartz.utils.QuartzUtils;
import org.quartz.CronTrigger;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;

/**
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class QuartzExternalListener extends AbstractExternalListener {

	/*
	 * (non-Javadoc)
	 * @see org.ow2.petals.component.framework.listener.AbstractExternalListener
	 * #start()
	 */
	@Override
	public void start() throws PEtALSCDKException {

		String jobName = QuartzUtils.generateJobName( getConsumes());

		Logger logger = getLogger();
		if( logger.isLoggable( Level.FINE ))
			logger.fine( "Defining a new job " + jobName + "." );

		Scheduler scheduler = ((QuartzSe) getComponent()).getQuartzScheduler();

		// FIXME: should we define a SU manager and cache the result?
		String cronExpression = getExtensions().get( QuartzConstants.CRON_EXPRESSION );
		if( cronExpression != null )
			cronExpression = cronExpression.trim();

		if( logger.isLoggable( Level.FINEST ))
			logger.finest( "The CRON expression is: " + cronExpression );

		try {
			// Create the job
			JobDetail jobDetail = new JobDetail( jobName, Scheduler.DEFAULT_GROUP, EmitSignalJob.class );
			JobDataMap jobDataMap = new JobDataMap();
			jobDataMap.put( QuartzConstants.SCHEDULER_MANAGER, this );
			jobDetail.setJobDataMap( jobDataMap );
			if( logger.isLoggable( Level.FINE ))
				logger.fine( "The " + jobName + " job was successfully created." );

			// Schedule the job
			String triggerName = QuartzUtils.generateTriggerName( getConsumes());
			CronTrigger trigger = new CronTrigger(	triggerName, Scheduler.DEFAULT_GROUP, cronExpression );
			if( logger.isLoggable( Level.FINE ))
				logger.fine( "The CRON trigger was successfully created." );

			scheduler.scheduleJob( jobDetail,trigger );
			if( logger.isLoggable( Level.FINE ))
				logger.fine( "The CRON trigger was successfully applied to the " + jobName + " job." );

		} catch( SchedulerException e ) {
			throw new PEtALSCDKException( "The " + jobName + " job could not be associated with a CRON trigger.", e );

		} catch( ParseException e ) {
			throw new PEtALSCDKException( "A CRON trigger could not be created for the " + jobName + " job.", e );
		}
	}


	/*
	 * (non-Javadoc)
	 * @see org.ow2.petals.component.framework.listener.AbstractExternalListener
	 * #stop()
	 */
	@Override
	public void stop() throws PEtALSCDKException {

		// Remember, there is one external listener per "consumes" block
		String jobName = QuartzUtils.generateJobName( getConsumes());
		Scheduler scheduler = ((QuartzSe) getComponent()).getQuartzScheduler();
		try {
			scheduler.deleteJob( jobName, Scheduler.DEFAULT_GROUP );
			Logger logger = getLogger();
			if( logger.isLoggable( Level.FINEST ))
				logger.finest( "The " + jobName + " job was successfully removed." );

		} catch( SchedulerException e ) {
			throw new PEtALSCDKException( "The " + jobName + " job could not be removed.", e );
		}
	}
}
