/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.se.quartz.job;

import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;

import org.ow2.petals.commons.log.Level;
import org.ow2.petals.commons.log.PetalsExecutionContext;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.logger.Utils;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.se.quartz.QuartzConsumeFlowStepBeginLogData;
import org.ow2.petals.se.quartz.QuartzExternalListener;
import org.ow2.petals.se.quartz.utils.QuartzConstants;
import org.ow2.petals.se.quartz.utils.QuartzUtils;
import org.quartz.Job;
import org.quartz.JobDataMap;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import com.ebmwebsourcing.easycommons.lang.StringHelper;
import com.ebmwebsourcing.easycommons.thread.ExecutionContext;

/**
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class EmitSignalJob implements Job {

    /*
     * (non-Javadoc)
     * 
     * @see org.quartz.Job #execute(org.quartz.JobExecutionContext)
     */
    public void execute(JobExecutionContext jobContext) throws JobExecutionException {

        // Get the external listener and the information it contains
        JobDataMap dataMap = jobContext.getJobDetail().getJobDataMap();
        QuartzExternalListener quartzExternalListener = (QuartzExternalListener) dataMap
                .get(QuartzConstants.SCHEDULER_MANAGER);
        Consumes consume = quartzExternalListener.getConsumes();
        ConfigurationExtensions extensions = new ConfigurationExtensions(consume.getAny());
        Logger logger = quartzExternalListener.getLogger();

        // Invoke the service
        String serviceIdentifier = QuartzUtils.buildTargetServiceIdentifier(consume);
        PetalsExecutionContext.initFlowInstanceId();
        PetalsExecutionContext.nextFlowStepId();
        logger.log(
                Level.MONIT,
                "",
                new QuartzConsumeFlowStepBeginLogData(ExecutionContext.getProperties().getProperty(
                        PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME), ExecutionContext
                        .getProperties()
.getProperty(
                                PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME), StringHelper
                        .nonNullValue(consume.getInterfaceName()), StringHelper
                        .nonNullValue(consume.getServiceName()), StringHelper.nonNullValue(consume
                        .getEndpointName()), StringHelper.nonNullValue(consume.getOperation()),
                        String.valueOf(jobContext.getFireTime().getTime())));
        try {
            Exchange exchange = quartzExternalListener.createConsumeExchange(consume,
                    MEPConstants.IN_ONLY_PATTERN);
            String content = extensions.get(QuartzConstants.CONTENT_EXPRESSION);
            if (content != null)
                content = content.trim();

            if (logger.isLoggable(Level.FINE)) {
                String msg = "Creating the scheduled message " + exchange.getExchangeId() + " for "
                        + serviceIdentifier + ".";
                logger.fine(msg);
            }

            if (logger.isLoggable(Level.FINEST))
                logger.finest(exchange.getExchangeId() + ": the sent content is\n" + content);

            exchange.getInMessage().setContent(SourceUtil.createSource(content));
            quartzExternalListener.send(exchange);

            if (logger.isLoggable(Level.INFO)) {
                String msg = "The scheduled message " + exchange.getExchangeId()
                        + " was successfully sent to " + serviceIdentifier + ".";
                logger.info(msg);
            }

        } catch (final MessagingException e) {
            if (logger.isLoggable(Level.INFO))
                logger.info("A scheduled message cound not be sent to " + serviceIdentifier
                        + " (MessagingException).");
            Utils.addMonitFailureTrace(logger,
					PetalsExecutionContext.getFlowAttributes(),
					e.getMessage(), Role.CONSUMER);

        } catch (final PEtALSCDKException e) {
            if (logger.isLoggable(Level.INFO))
                logger.info("A scheduled message cound not be sent to " + serviceIdentifier
                        + " (PEtALSCDKException).");
            Utils.addMonitFailureTrace(logger,
					PetalsExecutionContext.getFlowAttributes(),
					e.getMessage(), Role.CONSUMER);
        }
    }
}
